@@ -9,24 +9,28 @@ Created on Tue Jan 3 15:00:00 2023
 import os
 
 from aind_segmentation_evaluation.evaluate import run_evaluation
+from aind_segmentation_evaluation.graph_routines import volume_to_graph
+from tifffile import imread
 
 
 if __name__ == "__main__":
 
     # Initializations
-    shape = (148, 226, 282)
     data_dir = "./resources"
-    path_to_pred_volume = os.path.join(data_dir, "pred_volume.tif")
-    path_to_target_volume = os.path.join(data_dir, "target_volume.tif")
     target_graphs_dir = os.path.join(data_dir, "target_graphs")
+    path_to_target_labels = os.path.join(data_dir, "target_labels.tif")
+    pred_labels = imread(os.path.join(data_dir, "pred_labels.tif"))
+    pred_graphs = volume_to_graph(pred_labels)
 
     # Evaluation
     stats = run_evaluation(
-        shape,
-        target_graphs_dir=target_graphs_dir,
-        path_to_pred_volume=path_to_pred_volume,
+        target_graphs_dir,
+        path_to_target_labels,
+        pred_graphs,
+        pred_labels,
         output="tif",
         output_dir=data_dir,
+        filetype="tif",
     )
 
     # Write out results
