@@ -9,17 +9,60 @@ Created on Wed Dec 10 19:00:00 2022
 
 import os
 from random import sample
-
 import networkx as nx
 import numpy as np
 from scipy.ndimage.morphology import grey_dilation
 from skimage.morphology import skeletonize_3d
-
 from aind_segmentation_evaluation.utils import get_idx, get_xyz
 
 
 # Conversion Routines
-def graph_to_volume(list_of_graphs, shape):
+def to_world(idx, permute, scale, shift):
+    """ "
+    Converts "idx" to real-world coordinates.
+
+    Parameters
+    ----------
+        idx : list[idx]
+            Image indexes to be converted.
+        permute :
+            Permutation that is applied to "idx".
+        scale : list[float]
+            Scaling factor that is applied to permuted "idx".
+        shift : list[float]
+            Shift that is applied to "idx".
+
+    Returns
+    -------
+    list
+        The result of applying this series of transformations to "idx".
+
+    """
+    xyz = [idx[i] + shift[i] for i in permute]
+    xyz = [xyz[i] * scale[i] for i in range(3)]
+    return xyz
+
+
+def apply_permutation(my_list, permute):
+    """
+    Applies a permutation to a list.
+
+    Parameters
+    ----------
+    my_list : list
+        List of any type of values
+    permute : list[int]
+        Permutation that is applied to "my_list"
+
+    Returns
+    list
+        Permutation of input "my_list".
+
+    """
+    return [my_list[i] for i in permute]
+
+
+def graph_to_volume(list_of_graphs, shape, sparse=True):
     """
     Converts "list_of_graphs" to a sparse image volume.
 
@@ -39,8 +82,8 @@ def graph_to_volume(list_of_graphs, shape):
     num_dilations = 3
     volume = graph_to_skeleton(list_of_graphs, shape)
     for _ in range(num_dilations):
-        volume = grey_dilation(volume, mode="constant", size=(3, 3, 3))
-    return volume_to_dict(volume)
+        volume = grey_dilation(volume, mode="constant", cval=0, size=(3, 3, 3))
+    return volume_to_dict(volume) if sparse else volume
 
 
 def graph_to_skeleton(list_of_graphs, shape):
@@ -67,7 +110,13 @@ def graph_to_skeleton(list_of_graphs, shape):
     return volume
 
 
-def graph_to_swc(graph, path):
+def graph_to_swc(
+    graph,
+    path,
+    permute=[0, 1, 2],
+    scale=[1, 1, 1],
+    shift=[0, 0, 0],
+):
     """
     Converts graph to an swc file.
 
@@ -77,6 +126,12 @@ def graph_to_swc(graph, path):
         Graph which represents a neuron.
     path : str
         Path that swc file will be written to.
+    permute : list[int]
+        Permutation from image to real-world coordinates.
+    scale : list[int]
+        Scaling factor from image to real-world coordinates.
+    shift : list[float]
+            Shift that is applied to "idx".
 
     Returns
     -------
@@ -90,7 +145,9 @@ def graph_to_swc(graph, path):
     reindex = dict()
     while len(queue) > 0:
         parent, child = queue.pop(0)
-        swc.append(get_swc_entry(get_xyz(graph, child), 2, parent))
+        entry = to_world(get_xyz(graph, child), permute, scale, shift)
+        entry.extend([2, int(parent)])
+        swc.append(entry)
         visited.add(child)
         reindex[child] = len(swc)
         for nb in list(graph.neighbors(child)):
@@ -137,7 +194,7 @@ def skeleton_to_graph(skel):
     return graph
 
 
-def swc_to_graph(swc_dir, shape):
+def swc_to_graph(swc_dir, shape, permute=[0, 1, 2], scale=[1, 1, 1]):
     """
     Converts directory of swc files to a list of graphs.
 
@@ -147,6 +204,8 @@ def swc_to_graph(swc_dir, shape):
         Path to directory containing swc files.
     shape : tuple
         Dimensions of image volume in the order of (x,y,z).
+    scale : List[int]
+        Image to real-world coordinates scaling factors for [x, y, z].
 
     Returns
     -------
@@ -166,7 +225,7 @@ def swc_to_graph(swc_dir, shape):
                 parts = line.split()
                 child = int(parts[0])
                 parent = int(parts[-1])
-                xyz = read_xyz(parts[2:5])
+                xyz = read_xyz(parts[2:5], scale, permute)
                 idx = read_idx(xyz, shape)
                 graph.add_node(child, xyz=xyz, idx=idx)
                 if parent != -1:
@@ -198,7 +257,7 @@ def volume_to_dict(volume):
     return d
 
 
-def volume_to_graph(volume):
+def volume_to_graph(volume, min_branch_length=10):
     """
     Converts image to a list of graphs, where each label in the image
     corresponds to a distinct graph.
@@ -220,7 +279,8 @@ def volume_to_graph(volume):
     for i in [i for i in np.unique(skeleton) if i != 0]:
         mask_i = (skeleton == i).astype(int)
         graph_i = skeleton_to_graph(mask_i)
-        list_of_graphs.append(graph_i)
+        pruned_graph_i = prune(graph_i, min_branch_length=min_branch_length)
+        list_of_graphs.append(pruned_graph_i)
     return list_of_graphs
 
 
@@ -247,9 +307,8 @@ def embed_graph(graph, volume, val, root=1):
         Image volume.
 
     """
-    volume[get_idx(graph, root)] = val
-    for (i, j) in nx.bfs_edges(graph, root):
-        volume[get_idx(graph, j)] = val
+    for i in graph.nodes:
+        volume[get_idx(graph, i)] = val
     return volume
 
 
@@ -297,13 +356,48 @@ def get_nb(xyz, vec):
 
     Returns
     -------
-    TYPE
+    tuple
         Neighbor of node with coordinates "xyz".
 
     """
     return tuple([int(sum(i)) for i in zip(xyz, vec)])
 
 
+def prune(graph, min_branch_length=10):
+    """
+    Prune short branches that contain a leaf node
+
+    Parameters
+    ----------
+    graph : networkx.graph
+        Graph that represents a neuron.
+    min_branch_length : int
+        Minimum branch length
+
+    """
+    leaf_nodes = [i for i in graph.nodes if graph.degree[i] == 1]
+    for leaf in leaf_nodes:
+        # Traverse branch from leaf
+        queue = [leaf]
+        visited = set()
+        hit_junction = False
+        while len(queue) > 0:
+            node = queue.pop(0)
+            nbs = list(graph.neighbors(node))
+            if len(nbs) > 2:
+                hit_junction = True
+                break
+            else:
+                visited.add(node)
+                nb = [nb for nb in nbs if nb not in visited]
+                queue.extend(nb)
+
+        # Check length of branch
+        if hit_junction and len(visited) <= min_branch_length:
+            graph.remove_nodes_from(visited)
+    return graph
+
+
 # SWC read/write routines
 def write_swc(path_to_swc, list_of_entries, color=None):
     """
@@ -336,32 +430,7 @@ def write_swc(path_to_swc, list_of_entries, color=None):
             f.write("\n")
 
 
-def get_swc_entry(xyz, radius, parent):
-    """
-    Gets text (i.e. "entry") that will be written to an swc file.
-
-    Parameters
-    ----------
-    xyz : tuple
-        (x,y,z) coordinates of node corresponding to this entry.
-    radius : int
-        Size of node written to swc file.
-    parent : int
-        Parent of node corresponding to this entry.
-
-    Returns
-    -------
-    entry : str
-        Text (i.e. "entry") that will be written to an swc file.
-
-    """
-    entry = [val * get_scaling_factor() for val in xyz]
-    entry.reverse()
-    entry.extend([radius, int(parent)])
-    return entry
-
-
-def read_xyz(zyx):
+def read_xyz(xyz, scale, permute):
     """
     Reads the (z,y,x) coordinates from an swc file, then reverses and scales
     them.
@@ -370,6 +439,8 @@ def read_xyz(zyx):
     ----------
     zyx : str
         (z,y,x) coordinates.
+    scaling_factors : List[int]
+        Image to real-world coordinates scaling factors for [x, y, z].
 
     Returns
     -------
@@ -377,9 +448,10 @@ def read_xyz(zyx):
         The (x,y,z) coordinates from an swc file.
 
     """
-    zyx_scaled = [float(val) / get_scaling_factor() for val in zyx]
-    zyx_scaled.reverse()
-    return zyx_scaled
+    xyz = list(map(float, xyz))
+    xyz = [xyz[i] for i in permute]
+    xyz = [xyz[i] / scale[i] for i in range(3)]
+    return xyz
 
 
 def read_idx(xyz, shape):
@@ -422,16 +494,3 @@ def intergize(val, dim):
     """
     idx = min(max(np.round(val), 0), dim)
     return int(idx)
-
-
-def get_scaling_factor():
-    """
-    Gets scaling factor used in Janelia Workstation
-
-    Returns
-    -------
-    float
-        Scaling factor.
-
-    """
-    return 1.1010572351571979
